<?php
/*
Plugin Name: Store Info Plugin
Description: This WordPress plugin facilitates store management by enabling administrators to add, edit, and delete store information. It offers a Stored Information List feature with pagination, displaying details like store names, addresses, and statuses. Additionally, users can search for the nearest stores based on their current coordinates, with the program calculating distances. Administrators can add new stores through a form and edit existing information. The plugin includes shortcode support for embedding the nearest store search form on any page or post. With security measures in place to sanitize input data, it provides a seamless solution for store management within the WordPress environment.
Version: 1.0.01
Author: Panakj Yadav
Author URI: https://websquadz.com/
*/

function store_info_menu() {
    add_menu_page(
        'Store Info', // Page title
        'Store Info', // Menu title
        'manage_options', // Capability required
        'store-info', // Menu slug
        'store_info_page' // Callback function
    );
}
add_action('admin_menu', 'store_info_menu');

function store_info_page() {
    ?>
    <style>.store-information-form{max-width:400px;margin:0 auto;background-color:#f9f9f9;padding:20px;border-radius:5px;border:1px solid #ccc}.form-group{margin-bottom:20px}label{font-weight:700}.form-control{width:100%;padding:8px;border:1px solid #ccc;border-radius:4px;box-sizing:border-box}.btn{background-color:#007bff;color:#fff;border:none;padding:10px 20px;cursor:pointer;border-radius:4px}.btn:hover{background-color:#0056b3}
           form label{display:block;margin-bottom:10px;font-weight:700}input[type=text]{width:100%;padding:8px;margin-bottom:20px;border:1px solid #ccc;border-radius:4px;box-sizing:border-box}input[type=submit]{padding:10px 20px;background-color:#0073aa;color:#fff;border:none;border-radius:4px;cursor:pointer}input[type=submit]:hover{background-color:#005d80}
    </style>
    <hr>
     <div class="wrap">
        <h2>Add Store Information</h2>
        <div style="width:50%;float:left;">
                <form method="post" action="">
                    <!-- Display form fields with existing store information -->
                    <label for="store_name">Store Name:</label>
                    <input type="text" id="store_name" name="store_name" ><br>
                    <label for="address">Address:</label>
                    <input type="text" id="address" name="address" ><br>
                    <label for="coordinates">Latitude:</label>
                    <input type="text" id="coordinates" name="coordinates" ><br>
                    <label for="coordinates2">Longitude:</label>
                    <input type="text" id="coordinates2" name="coordinates2" ><br>
                    <label for="status">Status</label>
                    <select name="status" id="status">
                        <option value="open" selected>Open</option>
                        <option value="close">Close</option>
                    </select>
                    <br><br>
                    <!-- Add more fields as needed -->
                    <!-- <input type="hidden" name="store_id" > -->
                    <input type="submit" name="add_store" value="Add">
                </form>
        </div>
        <div style="width:50%;float:left;">
        <p style="text-align:center;">Download Sample csv file: <a href="https://sainikcanteen.in/wp-content/uploads/2024/04/demo-data-final.csv">here</a></p>
        <br>
            <form method="post" action="" class="store-information-form" enctype="multipart/form-data">
                <div class="form-group">
                    <label for="csv_file">Upload CSV File:</label><br>
                    <input type="file" id="csv_file" name="csv_file">
                </div>
                <input type="submit" name="upload_csv" value="Upload CSV" class="button button-primary">
            </form>
        </div>
    </div>
    <?php
}

function save_store_info() {
    if(isset($_POST['add_store'])){
        $store_name = sanitize_text_field($_POST['store_name']);
        $address = sanitize_textarea_field($_POST['address']);
        $coordinates = sanitize_text_field($_POST['coordinates']);
        $coordinates2 = sanitize_text_field($_POST['coordinates2']);
        $status = sanitize_text_field($_POST['status']);
        // Get existing stored stores
        $stored_stores = get_option('stored_stores', array());
    
        // Add new store information to the array
        $stored_stores[] = array(
            'store_name' => $store_name,
            'address' => $address,
            'coordinates' => $coordinates,
            'coordinates2' => $coordinates2,
            'status' => $status
        );
        // Update the option with the new array of stored stores
        update_option('stored_stores', $stored_stores);
    }
    
     if (isset($_POST['upload_csv'])) {
        // Handle CSV file upload and processing
        if ($_FILES['csv_file']['error'] == UPLOAD_ERR_OK && is_uploaded_file($_FILES['csv_file']['tmp_name'])) {
            $file = $_FILES['csv_file']['tmp_name'];
            $handle = fopen($file, "r");
            $header = true;
            while (($data = fgetcsv($handle, 1000, ",")) !== FALSE) {
                if ($header) {
                    $header = false;
                    continue;
                }
                // Extract data from CSV
                $storeID = sanitize_text_field($data[0]);
                $store_name = sanitize_textarea_field($data[1]);
                $address = sanitize_textarea_field($data[2]);
                $coordinates = sanitize_text_field($data[3]);
                $coordinates2 = sanitize_text_field($data[4]);

                $pkg = sanitize_text_field($data[8]);
                $status = sanitize_text_field($data[9]);
                $district = sanitize_textarea_field($data[5]);
                $state = sanitize_textarea_field($data[6]);
                $pincode = sanitize_textarea_field($data[7]);
        // Retrieve existing stored information array or initialize an empty array
        $stored_stores = get_option('stored_stores', array());
        // Add new store information to the array
        $stored_stores[] = array(
            'storeID' => $storeID,
            'store_name' => $store_name,
            'address' => $address,
            'coordinates' => $coordinates,
            'coordinates2' => $coordinates2,
            'status' => $status,
            'district'=>$district,
            'state'=>$state,
            'pincode'=>$pincode,
            'pkg'=>$pkg
        );
        // Update the option with the new array of stored stores
        update_option('stored_stores', $stored_stores);
                
    }
            fclose($handle);
            echo '<div class="updated"><p>CSV file uploaded and data saved!</p></div>';
        } else {
            echo '<div class="error"><p>Failed to upload CSV file.</p></div>';
        }
    }
    
}
add_action('admin_init', 'save_store_info');
function store_info_list_page() {
    // Check if the form is submitted
    if (isset($_POST['store_name'])) {
        $store_name = sanitize_text_field($_POST['store_name']);
    }

    if (isset($_POST['delete_all'])) {
        // Delete all stored information
        update_option('stored_stores', array());
        // Redirect back to the list page after deletion
        wp_redirect(admin_url('admin.php?page=stored-information-list'));
        exit;
    }
    $stored_stores = get_option('stored_stores', array());
    $total_items = count($stored_stores);
    $items_per_page = 15; // Number of items to display per page
    $current_page = isset($_GET['paged']) ? max(1, intval($_GET['paged'])) : 1;
    $offset = ($current_page - 1) * $items_per_page;
    $stored_stores_chunk = array_slice($stored_stores, $offset, $items_per_page);
    ?>
    <div class="wrap">
        <h2>Stored Information List</h2>
        <!-- Delete All Form -->
        <form method="post" action="" style="width: 100px;float: right;margin-bottom: 20px;">
            <input type="hidden" name="delete_all" value="1">
            <input type="submit" value="Delete All">
        </form>
        <br>
        <?php if (!empty($stored_stores_chunk)) : ?>
            <!-- Display Stored Stores -->
            <table class="wp-list-table widefat fixed striped">
                <thead>
                    <tr>
                        <th>Store id</th>
                        <th>Store Name</th>
                        <th>District</th>
                        <th>State</th>
                        <th>Pincode</th>
                        <th>Address</th>
                        <th>Coordinates</th>
                        <th>Coordinates 2</th>
                        <th>Status</th>
                        <th>Action</th>
                    </tr>
                </thead>
                <tbody>
                    <?php foreach ($stored_stores_chunk as $index => $store) : ?>
                        <tr>
                            <td><?php echo esc_html($store['storeID']); ?></td>
                            <td><?php echo esc_html($store['store_name']); ?></td>
                            <td><?php echo esc_html($store['district']); ?></td>
                            <td><?php echo esc_html($store['state']); ?></td>
                            <td><?php echo esc_html($store['pincode']); ?></td>
                            <td><?php echo esc_html($store['address']); ?></td>
                            <td><?php echo esc_html($store['coordinates']); ?></td>
                            <td><?php echo esc_html($store['coordinates2']); ?></td>
                            <td><?php echo esc_html($store['status']); ?></td>
                            <td>
                                <a href="<?php echo admin_url('admin.php?page=edit-stored-information&id=' . ($index + $offset)); ?>" class="button button-primary">Edit</a>
                                <br><br><a href="<?php echo admin_url('admin-post.php?action=delete_stored_information&id=' . ($index + $offset)); ?>" class="button button-danger" onclick="return confirm('Are you sure you want to delete this entry?')">Delete</a>
                            </td>
                        </tr>
                    <?php endforeach; ?>
                </tbody>
            </table>

            <?php
            $total_pages = ceil($total_items / $items_per_page);
            echo '<div class="tablenav">';
            echo '<div class="tablenav-pages">';
            echo paginate_links(array(
                'base' => admin_url('admin.php?page=stored-information-list&paged=%#%'),
                'format' => '&paged=%#%',
                'current' => $current_page,
                'total' => $total_pages,
            ));
            echo '</div>';
            echo '</div>';
            ?>
        <?php else : ?>
            <p>No stored information available.</p>
        <?php endif; ?>
    </div>
    <?php
}


function register_custom_admin_actions() {
    add_action('admin_post_delete_stored_information', 'delete_stored_information_action');
}
add_action('admin_init', 'register_custom_admin_actions');
function delete_stored_information_action() {
    // Check if the user has the necessary permissions
    if (!current_user_can('manage_options')) {
        wp_die('Unauthorized access', 'Error');
    }
    // Check if ID parameter is set in the URL
    if (isset($_GET['id'])) {
        $id = intval($_GET['id']); // Convert ID to integer
        $stored_stores = get_option('stored_stores', array());

        // Check if the ID is valid and exists in the stored_stores array
        if ($id >= 0 && $id < count($stored_stores)) {
            // Remove the store entry from the stored_stores array
            unset($stored_stores[$id]);
            // Update the option with the modified array of stored stores
            update_option('stored_stores', $stored_stores);
            
        }
    }

    // Redirect back to the list page after deletion
    wp_redirect(admin_url('admin.php?page=stored-information-list'));
    exit;
}
// Define the callback function for the edit stored information page
function edit_stored_information_page() {
    // Check if store ID is provided
    if (isset($_GET['id'])) {
        $store_id = intval($_GET['id']); // Sanitize store ID
        // Retrieve the stored stores
        $stored_stores = get_option('stored_stores', array());
        // Check if the store with the provided ID exists
        if (isset($stored_stores[$store_id])) {
            $store = $stored_stores[$store_id]; // Get store information
            ?>
           <style>
           form label{display:block;margin-bottom:10px;font-weight:700}input[type=text]{width:100%;padding:8px;margin-bottom:20px;border:1px solid #ccc;border-radius:4px;box-sizing:border-box}input[type=submit]{padding:10px 20px;background-color:#0073aa;color:#fff;border:none;border-radius:4px;cursor:pointer}input[type=submit]:hover{background-color:#005d80}

           </style>
            <div class="wrap">
                <h2>Edit Stored Information</h2>
                <form method="post" action="">
                    <!-- Display form fields with existing store information -->
                    storeID
                    <label for="store_name">Store ID:</label>
                    <input type="text" id="storeID" name="storeID" value="<?php echo esc_attr($store['storeID']); ?>"><br>

                    <label for="store_name">Store Name:</label>
                    <input type="text" id="store_name" name="store_name" value="<?php echo esc_attr($store['store_name']); ?>"><br>

                    <label for="store_name">District:</label>
                    <input type="text" id="district" name="district" value="<?php echo esc_attr($store['district']); ?>"><br>

                    <label for="store_name">State:</label>
                    <input type="text" id="state" name="state" value="<?php echo esc_attr($store['state']); ?>"><br>

                    <label for="store_name">Pincode:</label>
                    <input type="text" id="pincode" name="pincode" value="<?php echo esc_attr($store['pincode']); ?>"><br>

                    <label for="address">Address:</label>
                    <input type="text" id="address" name="address" value="<?php echo esc_attr($store['address']); ?>"><br>
                    <label for="coordinates">Latitude:</label>
                    <input type="text" id="coordinates" name="coordinates" value="<?php echo esc_attr($store['coordinates']); ?>"><br>
                    <label for="coordinates2">Longitude:</label>
                    <input type="text" id="coordinates2" name="coordinates2" value="<?php echo esc_attr($store['coordinates2']); ?>"><br>
                    <label for="pkg">PKG:</label>
                    <input type="text" id="pkg" name="pkg" value="<?php echo esc_attr($store['pkg']); ?>"><br>
                    
                    <select name="status" id="status">
                        <option value="open" value="<?php if($store['status']=='open'){echo 'selected';} ?>">Open</option>
                        <option value="close" value="<?php if($store['status']=='close'){echo 'selected';} ?>">Close</option>
                    </select>
                    <br> <br>
                    <!-- Add more fields as needed -->
                    <input type="hidden" name="store_id" value="<?php echo $store_id; ?>">
                    <input type="submit" name="update_store" value="Update">
                </form>
            </div>
<?php

if (isset($_POST['update_store'])) {
    $store_id = intval($_POST['store_id']); // Sanitize store ID
    $storeID = sanitize_text_field($_POST['storeID']);
    $store_name = sanitize_text_field($_POST['store_name']);

    $district = sanitize_text_field($_POST['district']);
    $state = sanitize_text_field($_POST['state']);
    $pincode = sanitize_text_field($_POST['pincode']);
    $address = sanitize_text_field($_POST['address']);
    $coordinates = sanitize_text_field($_POST['coordinates']);
    $coordinates2 = sanitize_text_field($_POST['coordinates2']);
    $pkg = sanitize_text_field($_POST['pkg']);

    // Retrieve the stored stores
    $stored_stores = get_option('stored_stores', array());

    // Update store information
    if (isset($stored_stores[$store_id])) {
        $stored_stores[$store_id]['store_name'] = $store_name;
        $stored_stores[$store_id]['district'] = $district;
        $stored_stores[$store_id]['state'] = $state;
        $stored_stores[$store_id]['pincode'] = $pincode;
        $stored_stores[$store_id]['address'] = $address;
        $stored_stores[$store_id]['coordinates'] = $coordinates;
        $stored_stores[$store_id]['coordinates2'] = $coordinates2;
        $stored_stores[$store_id]['pkg'] = $pkg;
        // Update the stored stores option
        update_option('stored_stores', $stored_stores);
        // Redirect back to the list page after updating
        wp_redirect(admin_url('admin.php?page=stored-information-list'));
        exit;
    }
}

?>
<?php
        } else {
            echo '<p>Store not found.</p>';
        }
    } else {
        echo '<p>Store ID not provided.</p>';
    }
}

// Register the admin page for editing stored information
function register_edit_stored_information_page() {
    add_submenu_page(
        null,
        'Edit Stored Information',
        'Edit Stored Information',
        'manage_options',
        'edit-stored-information',
        'edit_stored_information_page'
    );
}
add_action('admin_menu', 'register_edit_stored_information_page');


function store_info_list_menu() {
    add_submenu_page(
        'store-info',
        'Stored Information List',
        'Stored Information List',
        'manage_options',
        'stored-information-list',
        'store_info_list_page'
    );

    add_submenu_page(
        'store-info',
        'Search Nearest Store',
        'Search Nearest Store',
        'manage_options',
        'search-nearest-store',
        'search_nearest_store_page'
    );
}
add_action('admin_menu', 'store_info_list_menu');

function search_nearest_stores($coordinates1, $coordinates2, $stored_stores, $distance_range) {
   
    // Calculate distances to all stores
    $distances = array();
    foreach ($stored_stores as $store) {
        $faraway = calculate_distance($coordinates1, $coordinates2, $store['coordinates'], $store['coordinates2']);
        $distances[] = array(
            'farAway' => $faraway,
            'store_name' => $store['store_name'],
            'district' => $store['district'],
            'state' => $store['state'],
            'pincode' => $store['pincode'],
            'storeID' => $store['storeID']
        );
    }
    // Sort stores by distance
    usort($distances, function($a, $b) {
        return $a['farAway'] - $b['farAway'];
    });
   // $nearest_stores = array_slice($distances, 0, $limit);
   $range = $distance_range;
    // Filter the array based on the range
    $nearest_stores = array_filter($distances, function($store) use ($range) {
        return $store['farAway'] < $range;
    });

    return $nearest_stores;
}

function calculate_distance($lat1, $lon1, $lat2, $lon2) {
    $earth_radius = 6371; // Radius of the Earth in kilometers
    // Convert latitude and longitude from degrees to radians
    $lat1 = deg2rad($lat1);
    $lon1 = deg2rad($lon1);
    $lat2 = deg2rad($lat2);
    $lon2 = deg2rad($lon2);
    // Calculate the differences between the coordinates
    $delta_lat = $lat2 - $lat1;
    $delta_lon = $lon2 - $lon1;
    // Haversine formula
    $a = sin($delta_lat / 2) * sin($delta_lat / 2) + cos($lat1) * cos($lat2) * sin($delta_lon / 2) * sin($delta_lon / 2);
    $c = 2 * atan2(sqrt($a), sqrt(1 - $a));
    $distance = $earth_radius * $c;
    return round($distance, 2);
}

//search page
function search_nearest_store_page() {
    if (isset($_POST['latitude']) && isset($_POST['longitude'])) {
        $latitude = sanitize_text_field($_POST['latitude']);
        $longitude = sanitize_text_field($_POST['longitude']);
        $stored_stores = get_option('stored_stores', array());
        $distance_range = isset($_POST['distance']) ? intval($_POST['distance']) : 5; // Default range set to 50km
       // print_r($stored_stores); die();
        $nearest_stores = search_nearest_stores($latitude, $longitude, $stored_stores, $distance_range);
        //echo $distance_range;
        ?>
        <div class="wrap">
            <h2>Search Nearest Store</h2>
                <form method="post" action="">
                    <div style="display: flex; flex-direction: row;">
                        <div style="margin-right: 20px;">
                            <label for="latitude">Latitude:</label>
                            <input type="text" required id="latitude" name="latitude" value="<?php echo isset($latitude) ? esc_attr($latitude) : ''; ?>"><br>
                        </div>
                        <div>
                            <label for="longitude">Longitude:</label>
                            <input type="text" required id="longitude" name="longitude" value="<?php echo isset($longitude) ? esc_attr($longitude) : ''; ?>"><br>
                        </div>
                        &nbsp;&nbsp;
                        <div>
                            <label for="distance">Distance:</label><br>
                            <select name="distance" required>
                                <option value="5" <?php echo ($distance_range == 5) ? 'selected' : ''; ?>>5km</option>
                                <option value="10" <?php echo ($distance_range == 10) ? 'selected' : ''; ?>>10km</option>
                                <option value="25" <?php echo ($distance_range == 25) ? 'selected' : ''; ?>>25km</option>
                            <option value="50" <?php echo ($distance_range == 50) ? 'selected' : ''; ?>>50km</option>
                            <option value="100" <?php echo ($distance_range == 100) ? 'selected' : ''; ?>>100km</option>
                            </select>

                            <br>
                        </div>
                    </div>
                    <br>
                    <input type="submit" value="Search">
                </form>
            <?php if (!empty($nearest_stores)) : ?>
                <h3>Nearest Stores:<small> ( Within <?= $distance_range ?> KM)</small> </h3>
                <table>
                    <thead>
                        <tr>
                            <th>S.N.</th>
                            <th>Store ID</td>
                            <th>Store Name</th>
                            <th>City</th>
                            <th>State</th>
                            <th>Zip Code</th>
                            <th>Distance</th>
                        </tr>
                    </tead>
                    <tbody>
                    <?php 
                    $count = 1;
                    foreach ($nearest_stores as $row) : ?>
                        <tr>
                            <td><?= $count; ?></td>
                            <td><?php echo esc_html($row['storeID']); ?></td>
                            <td><?php echo esc_html($row['store_name']); ?></td>
                            <td><?php echo esc_html($row['district']); ?></td>
                            <td><?php echo esc_html($row['state']); ?></td>
                            <td><?php echo esc_html($row['pincode']); ?></td>
                            <td><?php echo esc_html($row['farAway']); ?> km</td>
                        </tr>
                    <?php  $count++; endforeach; ?>
                    </tbody>
                </table>
            
            <?php else : ?>
                <p>No nearest stores found.</p>
            <?php endif; ?>
        </div>
        <?php
    } else {
        ?>
        <div class="wrap">
            <h2>Search Nearest Store</h2>
                <form method="post" action="">
                    <div style="display: flex; flex-direction: row;">
                        <div style="margin-right: 20px;">
                            <label for="latitude">Latitude:</label>
                            <input type="text" id="latitude" name="latitude"required ><br>
                        </div>
                        <div>
                            <label for="longitude">Longitude:</label>
                            <input type="text" id="longitude" name="longitude" required><br>
                        </div>
                        &nbsp;&nbsp;
                        <div>
                            <label for="distance">Distance:</label><br>
                            <select name="distance" required>
                                <option value="5">5km</option>
                                <option value="10">10km</option>
                                <option value="25">25km</option>
                                <option value="50">50km</option>
                                <option value="100">100km</option>
                            </select>
                            <br>
                        </div>
                    </div>
                    <br>
                    <input type="submit" value="Search">
                </form>
        </div>
        <?php
    }
}
function save_stored_information() {
    if (isset($_POST['submit'])) {
        $store_name = sanitize_text_field($_POST['store_name']);
        $address = sanitize_textarea_field($_POST['address']);
        $coordinates = sanitize_text_field($_POST['coordinates']);
        $coordinates2 = sanitize_text_field($_POST['coordinates2']);
        $status = sanitize_text_field($_POST['status']);
        update_option('store_name', $store_name);
        update_option('address', $address);
        update_option('coordinates', $coordinates);
        update_option('coordinates2', $coordinates2);
        update_option('status', $status);
        echo '<div class="updated"><p>Store information updated!</p></div>';
    }
}
add_action('admin_init', 'save_stored_information');
// Define the shortcode function
function search_nearest_store_shortcode() {
    ob_start();
    search_nearest_store_page(); // Call the function to render the search form
    return ob_get_clean();
}
// Register the shortcode
add_shortcode('search_nearest_store', 'search_nearest_store_shortcode');
